import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.Timer;

/**
   Este programa muestra el uso de una barra de progreso (Qmetro)
   para monitorear el progreso de una tarea en un thread
*/
public class ProgressBarTest{  
   public static void main(String[] args){  
      ProgressBarFrame frame = new ProgressBarFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.show();
   }
}

/**
   Marco que contiene un botn para lanzar una barra de progreso con una 
   actividad simulada y el area de texto para la salida de la actividad
*/
class ProgressBarFrame extends JFrame{  
   private Timer activityMonitor;
   private JButton startButton;
   private JProgressBar progressBar;
   private JTextArea textArea;
   private SimulatedActivity activity;
   public static final int WIDTH = 300;
   public static final int HEIGHT = 200;   
   public ProgressBarFrame(){  
      setTitle("ProgressBarTest");
      setSize(WIDTH, HEIGHT);

      Container contentPane = getContentPane();

      // area de texto para la salida de la actividad
      textArea = new JTextArea();

      // panel con boton start y barra de progreso

      JPanel panel = new JPanel();
      startButton = new JButton("Start");
      progressBar = new JProgressBar();
      progressBar.setStringPainted(true);
      panel.add(startButton);
      panel.add(progressBar);
      contentPane.add(new JScrollPane(textArea), 
         BorderLayout.CENTER);
      contentPane.add(panel, BorderLayout.SOUTH); 

      // configurar la accin del boton

      startButton.addActionListener(new 
         ActionListener(){  
            public void actionPerformed(ActionEvent event){  
               progressBar.setMaximum(1000);
               activity = new SimulatedActivity(1000);
               activity.start();
               activityMonitor.start();
               startButton.setEnabled(false);
            }
         });


      // configurar la accin del temporizador

      activityMonitor = new Timer(500, new 
         ActionListener(){  
            public void actionPerformed(ActionEvent event){  
               int current = activity.getCurrent();
               
               // mostrar progreso
               textArea.append(current + "\n");
               progressBar.setValue(current);
               
               // checkear si la tarea ha sido completada
               if (current == activity.getTarget()) {  
                  activityMonitor.stop();
                  startButton.setEnabled(true);
               }
            }
         });
   }
}

class SimulatedActivity extends Thread{ 
   /**
      Construye el thread para simular actividad
      El thread incrementa un contador desde 0 hasta un valor informado t
   */
   private int current;
   private int target;    
   public SimulatedActivity(int t){  
      current = 0;
      target = t;
   }

   public int getTarget(){  
      return target;
   }

   public int getCurrent() {  
      return current;
   }

   public void run(){  
      try{
         while (current < target && !interrupted()){    
            sleep(100);
            current++;
         }
      }
      catch(InterruptedException e){ }
   }
}







