
/** 
 *  Este programa resuelve la corrupcion de datos 
 * que tenamos en UnSinchBankTest  
 * Adecuaciones. Ing. Tymoschuk, Jorge
*/
public class SynchBankTest01{ 
   public static final int NACCOUNTS = 10;  //  Cantidad de cuentas
   public static final int INITIAL_BALANCE = 10000; //  Balance inicial
   public static void main(String[] args){  
      Bank bank = new Bank(NACCOUNTS, INITIAL_BALANCE);
      int i;
      for (i = 0; i < NACCOUNTS; i++){  
         TransferThread t = new TransferThread(bank, 
            INITIAL_BALANCE/50);
         t.setPriority(Thread.NORM_PRIORITY + i % 2);
         t.start();
      } //  for
   }    //  main
}       //  class

class Bank{     //  Un banco y sus cuentas
   public static final int NTEST = 10000;
   private final int[] accounts;
   private long nTransacts = 0;
   private int nTests     = 0;
   private int contTransNoAte = 0;

   public Bank(int n, int initialBalance){  //    
      accounts = new int[n];
      int i;
      for (i = 0; i < accounts.length; i++)
         accounts[i] = initialBalance;
      nTransacts = 0;
   }

    public synchronized void transfer(int origen, int destino, int monto)
      throws InterruptedException{  //  transferencia de dinero 
        if(accounts[origen] < monto)
            contTransNoAte++;
        else{
            accounts[origen] -= monto;
            // int cont = 0;
            // for (int i=0;i<85000;i++)cont++; //  Separando debito/crdito 
            accounts[destino] += monto;
        }
        nTransacts++;
        notifyAll();    
        if (nTransacts % NTEST == 0) test();
   }

   public void test(){    //  Verificando integridad del objeto Bank
      int sum = 0, ctasNeg=0;String texto;
      for (int i = 0; i < accounts.length; i++){
         sum += accounts[i];
         if(accounts[i]<0)ctasNeg++;
      }   
      nTests++;
      texto = "Banco chequeado " + nTests + " veces, Suma: " + sum;
      texto+=", Ctas sdo neg. "+ctasNeg;
      texto+=", Trans no atend. "+contTransNoAte;
      contTransNoAte = 0;
      System.out.println(texto);
   }

   public int size(){return accounts.length;}
   
   public int getTests(){return nTests;}
}

class TransferThread extends Thread{  //  Hilo que comanda transferencias
   private Bank banco;
   private int montoMax;
   private static final int REPS = 1000;

   public TransferThread(Bank banco, int montoMax){  
      this.banco = banco;
      this.montoMax = montoMax;
   }

   public void run(){  
      try{  
         while (banco.getTests() < 15 ){  
            for (int i = 0; i < REPS; i++){
               int ctaOrigen  = (int)(banco.size() * Math.random());
               int ctaDestino = (int)(banco.size() * Math.random());
               int monto = (int)(montoMax * Math.random());
               banco.transfer(ctaOrigen, ctaDestino, monto);
               sleep(1);
            }   //  for
         }      //  while
      }         //  try
      catch(InterruptedException e) {}
   }            //  run
}






